<?php

namespace PaymentPlugins\PPCP\WooFunnels\Upsell\PaymentGateways;

use PaymentPlugins\PayPalSDK\Amount;
use PaymentPlugins\PayPalSDK\Order;
use PaymentPlugins\PayPalSDK\Refund;
use PaymentPlugins\WooCommerce\PPCP\Assets\AssetsApi;
use PaymentPlugins\WooCommerce\PPCP\Main;
use PaymentPlugins\WooCommerce\PPCP\PaymentHandler;
use PaymentPlugins\WooCommerce\PPCP\PaymentResult;
use PaymentPlugins\WooCommerce\PPCP\Utilities\NumberUtil;
use PaymentPlugins\WooCommerce\PPCP\Utilities\OrderLock;

class AbstractGateway extends \WFOCU_Gateway {

	public $id = '';

	public $refund_supported = true;

	protected $assets;

	protected $payment_handler;

	/**
	 * @var \WFOCU_Logger
	 */
	protected $logger;

	public function __construct( AssetsApi $assets, PaymentHandler $payment_handler, $logger ) {
		$this->assets          = $assets;
		$this->payment_handler = $payment_handler;
		$this->logger          = $logger;
	}

	public static function get_instance() {
		return Main::container()->get( static::class );
	}

	public function get_payment_method_script_handles() {
		return [];
	}

	public function process_charge( $order ) {
		$this->handle_client_error();
		$payment_method = $this->get_wc_gateway();
		$this->payment_handler->set_payment_method( $payment_method );
		$client = $this->payment_handler->client;
		try {
			$this->payment_handler->set_use_billing_agreement( true );
			$paypal_order = $client->orders->create( $this->get_create_order_params( $order ) );
			if ( is_wp_error( $paypal_order ) ) {
				throw new \Exception( $paypal_order->get_error_message() );
			}
			if ( Order::CAPTURE == $paypal_order->intent ) {
				OrderLock::set_order_lock( $order );
				$response = $client->orders->capture( $paypal_order->id, $this->payment_handler->get_payment_source( $order ) );
			} else {
				$response = $client->orders->authorize( $paypal_order->id, $this->payment_handler->get_payment_source( $order ) );
			}
			$result = new PaymentResult( $response, $order, $payment_method );
			if ( $result->success() ) {
				WFOCU_Core()->data->set( '_transaction_id', $result->get_capture_id() );

				return $this->handle_result( true );
			} else {
				throw new \WFOCU_Payment_Gateway_Exception( $result->get_error_message(), 400 );
			}
		} catch ( \Exception $e ) {
			throw new \WFOCU_Payment_Gateway_Exception( $e->getMessage() );
		}
	}

	public function handle_client_error() {
		$package = WFOCU_Core()->data->get( '_upsell_package' );
		if ( $package && isset( $package['_client_error'] ) ) {
			$this->logger->log( sprintf( 'PPCP client error: %s', sanitize_text_field( $package['_client_error'] ) ) );
		}
	}

	private function get_create_order_params( $order ) {
		$paypal_order  = $this->payment_handler->get_create_order_params( $order );
		$current_offer = WFOCU_Core()->data->get( 'current_offer' );
		if ( $current_offer ) {
			$paypal_order->getPurchaseUnits()->get( 0 )->invoice_id = sprintf( '%s-%s', $order->get_id(), $current_offer );
		}

		return $paypal_order;
	}

	/**
	 * @param \WC_Order $order
	 *
	 * @return bool|void
	 */
	public function process_refund_offer( $order ) {
		$transaction_id = isset( $_POST['txn_id'] ) ? $_POST['txn_id'] : false;
		$amount         = isset( $_POST['amt'] ) ? round( $_POST['amt'], 2 ) : false;
		if ( ! $transaction_id || ! $amount ) {
			return;
		}
		$client = $this->payment_handler->client->orderMode( $order );
		$result = $client->captures->refund( $transaction_id, new Refund( [
			'amount' => new Amount( [
				'value'         => NumberUtil::round( $amount ),
				'currency_code' => $order->get_currency()
			] )
		] ) );

		if ( is_wp_error( $result ) ) {
			$this->logger->log( sprintf( 'Transaction %s refunded: Amount: %s', $order->get_id(), $amount ) );

			return true;
		} else {
			$this->logger->log( sprintf( 'Error processing refund for order %s. Reason: %s', $order->get_id(), $result->get_error_message() ) );

			return false;
		}
	}

	public function get_transaction_link( $transaction_id, $order_id ) {
		$order = wc_get_order( $order_id );

		return $this->get_wc_gateway()->get_transaction_url( $order );
	}

}