<?php

namespace PaymentPlugins\WooCommerce\PPCP;

use PaymentPlugins\WooCommerce\PPCP\Utilities\OrderLock;
use PaymentPlugins\WooCommerce\PPCP\Utilities\PayPalFee;


class WebhookEventReceiver {

	private $client;

	private $payment_handler;

	private $logger;

	public function __construct(
		WPPayPalClient $client,
		PaymentHandler $payment_handler, Logger $logger
	) {
		$this->client          = $client;
		$this->payment_handler = $payment_handler;
		$this->logger          = $logger;
		$this->initialize();
	}

	private function initialize() {
		add_action( 'wc_ppcp_webhook_event_payment.capture.completed', [ $this, 'do_capture_completed' ], 10, 2 );
		add_action( 'wc_ppcp_webhook_event_payment.capture.refunded', [ $this, 'do_refund_processed' ], 10, 2 );
	}

	/**
	 * @param \PaymentPlugins\PayPalSDK\Capture      $capture
	 * @param \PaymentPlugins\PayPalSDK\WebhookEvent $event
	 */
	public function do_capture_completed( $capture, $event ) {
		//only process if the order doesn't have a lock
		$order = wc_get_order( $capture->invoice_id );
		if ( $order && ! OrderLock::has_order_lock( $order ) ) {
			$transaction_id = $order->get_transaction_id();
			if ( ! $transaction_id ) {
				$paypal_order_id = $order->get_meta( Constants::ORDER_ID );
				if ( $paypal_order_id ) {
					$paypal_order = $this->client->orderMode( $order )->orders->retrieve( $paypal_order_id );
					if ( ! is_wp_error( $paypal_order ) ) {
						$order->payment_complete( $capture->id );
						$this->payment_handler->save_order_meta_data( $order, $paypal_order );
					} else {
						throw new \Exception( $paypal_order->get_error_message(), 400 );
					}
				}
			}
		}
	}

	/**
	 * @param \PaymentPlugins\PayPalSDK\Refund       $refund
	 * @param \PaymentPlugins\PayPalSDK\WebhookEvent $event
	 *
	 * @throws \Exception
	 */
	public function do_refund_processed( $refund, $event ) {
		global $wpdb;
		$this->logger->info( print_r( $event, true ) );
		if ( isset( $refund->invoice_id ) ) {
			$order = wc_get_order( $refund->invoice_id );
			if ( $order && ! OrderLock::has_order_lock( $order ) ) {
				// check if the refund exists already
				$refund_id = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM {$wpdb->posts} AS posts LEFT JOIN {$wpdb->postmeta} AS meta ON posts.ID = meta.post_id WHERE posts.post_type = %s AND meta.meta_key = %s AND meta.meta_value = %s LIMIT 1",
					'shop_order_refund', Constants::PAYPAL_REFUND, $refund->id ) );
				// refund doesn't exist so create it
				if ( ! $refund_id ) {
					wc_create_refund( [
						'amount'   => $refund->amount->value,
						'reason'   => isset( $refund->note_to_payer ) ? $refund->note_to_payer : __( 'Refund created within PayPal.', 'pymntpl-paypal-woocommerce' ),
						'order_id' => $order->get_id()
					] );
					/**
					 * @var \WC_Order_Refund $wc_refund
					 */
					$wc_refund = Main::container()->get( RefundsManager::class )->refund;
					if ( $wc_refund ) {
						// update the net amount since the refund affects that
						PayPalFee::update_net( $refund, $order );
						$wc_refund->update_meta_data( Constants::PAYPAL_REFUND, $refund->id );
						$wc_refund->save();
						$order->add_order_note( sprintf( __( 'Order refunded in PayPal. Amount: %1$s. Refund ID: %2$s', 'pymntpl-paypal-woocommerce' ),
							wc_price( $refund->amount->value, [ 'currency' => $refund->amount->currency ] ), $refund->id ) );
					}
				}
			}
		}
	}

}